#!/usr/bin/python

##############################################################################################
# Copyright (c) by George Ruinelli 
# License: 	GPL3
# Description:	This application shows the angle in a graph.
#	    	It is primarly an example applications which shows the 
#		interactions between QML and PySide.
#		Also it shows how a graph can be created in Python and shown in QML
##############################################################################################


import sys
from PySide.QtCore import *
from PySide.QtGui import *
from PySide.QtDeclarative import *

import random
import math
import platform


SENSOR_PATH = "/sys/devices/platform/lis3lv02d/position"

try:
  f = open(SENSOR_PATH, 'r' )
  coords = f.readline()
  f.close()
  print "Sensor data:", coords
  print "OK, sensor is available, we seem to be on a N9 or N950 (or a compatible device)"
  Valid_Device=True
except:
  print "Could not get valid data from acceleration sensor. Are we on a N9 or N950 (or a compatible device)?"
  print "Acceleration data will be simulated!"
  Valid_Device=False
  
  

display_orientation_portrait=True

arrayDataLen=800
arrData=[[0]*arrayDataLen,[0]*arrayDataLen,[0]*arrayDataLen,[0]*arrayDataLen,[0]*arrayDataLen] #main data
arrayDataIndex=0	#points to latest used data field


correctionAngleA=+5 #angle corrections
correctionAngleB=-2





class Data():
  display_orientation_portrait=True
  
  
  
  

################################################################################################  

class Graph (QDeclarativeItem):
  def __init__(self, parent = None):
    QDeclarativeItem.__init__(self, parent)
    # need to disable this flag to draw inside a QDeclarativeItem
    self.setFlag(QGraphicsItem.ItemHasNoContents, False)
    self.width = 0
    self.height = 0



  #This function creates the whole graph
  def paint(self, paint, options, widget):
    global data    
    
    old_width=self.width
    if(data.display_orientation_portrait==True): 
      self.width=4*110
      self.height=320
    else: 
      self.width=4*200
      self.height=120
    

    paint.setRenderHints(QPainter.Antialiasing, True);

    paint.setPen(QColor("gray"))
    paint.setBrush(QColor("black"))
    paint.setFont(QFont('Decorative', 12))

    #show graph border, background
    paint.drawRect(0,0,self.width,2*self.height)           
     
   
    
    #Formating X axis
    #Small Y lines
    for i in range(self.height, 2*self.height, 10): 
      paint.drawLine(0,i,5,i) #1 line per degree
      paint.drawLine(0,2*self.height-i,5,2*self.height-i)
      paint.drawLine(self.width-5,i,self.width,i)
      paint.drawLine(self.width-5,2*self.height-i,self.width,2*self.height-i)
	
    #Big Y lines
    for i in range(self.height, 2*self.height, 50): 
      paint.drawLine(0,i,self.width,i) #line per degree
      paint.drawLine(0,2*self.height-i,self.width,2*self.height-i)
      paint.drawText(10, i-4, str((i-self.height)/10))
      paint.drawText(10, 2*self.height-i-4, str(-1*(i-self.height)/10))

    #Formating Y axis
    #Small X lines
    for i in range(self.width, 0, -40): 
      paint.drawLine(i,self.height-5,i,self.height+5)
      
    #Big X lines
    for i in range(0, self.width, 120): 
      paint.drawLine(self.width-i,self.height-20,self.width-i,self.height+20)
      if(i==0): j=i+25 
      else:j=i
      paint.drawText(self.width-j+5, self.height+20, str(i/4))

    #Zero line
    paint.setPen(QColor("white"))
    paint.drawLine(0,self.height,self.width,self.height) 




    #Draw graph   
    for i in range(0,self.width-1):        
      k1=(arrayDataIndex-i-1) % arrayDataLen
      k2=(k1+1) % arrayDataLen
      
      if(arrData[4][k2]=="P"): #Portrait angle
	paint.setPen(QColor("yellow")) 
      elif(arrData[4][k2]=="L"): #Landscape angle
	paint.setPen(QColor("green")) 
      else:
	continue #array entry empty. do not draw any data
      
      a1=int(arrData[3][k1]*-10)+self.height
      a2=int(arrData[3][k2]*-10)+self.height
      paint.drawLine(self.width-1-i, a1, self.width-1-i+1, a2)
	  
	  
    root.updateGraphWidth(self.width, 2*self.height)
    


    
    
    
################################################################################################    
    
class QML_to_Python_Interface(QObject):
    global data
    @Slot(str)
    def newOrientation(self, s):
        #print "New Orientation:", s
        if(s=="portrait"): data.display_orientation_portrait=True
        else: data.display_orientation_portrait=False



    
################################################################################################    
    
    
    
    
    
    
def getSensorData():
  global Valid_Device
  
  if(Valid_Device==True): #read out sensor
    f = open(SENSOR_PATH, 'r' )
    data=f.readline().split(",")
    coords=[int(data[0][1:]), int(data[1]), int(data[2][:-2])]
    f.close()
  else: #Sensor not available, use random data
    coords=[0,0,0]
    coords[0]=random.uniform(-500,500)
    coords[1]=random.uniform(-20,20)
    coords[2]=random.uniform(500, 1500)

  return coords    
    
    
    
    
    
    
def Process_Data():
  global arrData, arrayDataIndex, arrayDataLen
  global correctionAngleA, correctionAngleB
  global data
  
  smoothing=50
  coordinations_smoothed=[0, 0, 0]
  for i in range(smoothing):
    coordinations=getSensorData()
    coordinations_smoothed[0]=coordinations_smoothed[0] + coordinations[0]
    coordinations_smoothed[1]=coordinations_smoothed[1] + coordinations[1]
    coordinations_smoothed[2]=coordinations_smoothed[2] + coordinations[2]
  

  #Write it into data array
  arrayDataIndex=arrayDataIndex+1
  arrayDataIndex=arrayDataIndex % arrayDataLen
  
  arrData[0][arrayDataIndex]=coordinations_smoothed[0]/50
  arrData[1][arrayDataIndex]=coordinations_smoothed[1]/smoothing
  arrData[2][arrayDataIndex]=coordinations_smoothed[2]/smoothing


  
  AccX=float(arrData[0][arrayDataIndex])
  AccY=float(arrData[1][arrayDataIndex])
  AccZ=float(arrData[2][arrayDataIndex])


  #Division by zero prevention
  if(AccX==0): AccX=0.1
  if(AccY==0): AccY=0.1
  if(AccZ==0): AccZ=0.1
  
  AngleA=math.atan(AccX/AccZ)*-57.3+correctionAngleA
  AngleB=math.atan(AccY/AccZ)*-57.3+correctionAngleB

  #Save Angle in array for usage in graph
  if(data.display_orientation_portrait==True):
    arrData[3][arrayDataIndex]=AngleA *-1
    arrData[4][arrayDataIndex]="P"
  else:
    arrData[3][arrayDataIndex]=AngleB
    arrData[4][arrayDataIndex]="L"
    
  root.tellQMLToRedrawGraph()
  

    
    
    
    
    
    
    
    

if __name__ == '__main__':
    app = QApplication(sys.argv)

    #qmlRegisterType(NAME OF CLASS, 	QML IMPORT NAME, 	VERSION MAJOR, 	VERSION MINOR, 	COMPONENT NAME IN QML)
    qmlRegisterType(Graph, 'myAngleGraph', 1, 0, 'AngleGraph')

    view = QDeclarativeView()
    view.setSource(QUrl.fromLocalFile('qml/main.qml'))
    root = view.rootObject()



    
    # instantiate the Python object
    qml_to_python = QML_to_Python_Interface()
    # expose the object to QML
    context = view.rootContext()
    context.setContextProperty("qml_to_python", qml_to_python)
    
    
    
    if(platform.machine().startswith('arm')): view.showFullScreen()
    view.show()
    
    # constant timer
    app.timer_sampling = QTimer()
    QObject.connect(app.timer_sampling, SIGNAL("timeout()"), Process_Data)
    
    app.timer_sampling.start(250)
    
     
    data = Data()
    
    sys.exit(app.exec_())
