import QtQuick 1.0
import com.meego 1.0
import myAngleGraph 1.0



Page {
    
    //We want to tell Python when ever our display orientation changes
    //This is done with the signal "onWidthChanged".
    //This how ever is not called at start up, because of that we also use a sigle shot timer
    onWidthChanged: {
      tellPythonDisplayOrientation()
    }

    Timer {
	interval: 1; running: true; repeat: false
	onTriggered: {
	  tellPythonDisplayOrientation()    
	}
    }
    
    //Tell Python that the display orientation changed
    function tellPythonDisplayOrientation(){
	if (width > 600) { // landscape		
	    qml_to_python.newOrientation("landscape")	
	} else { // portrait		
	    qml_to_python.newOrientation("portrait")
	}
    }
    
    
//*******************************************************
    
    
    //Call from Python to resize the graph
    function updateGraphWidth2(width, height) {
        angleGraph01.width=width
        flickable.width=width
        angleGraph01.height=height
        flickable.height=height
    }

    //I dont know how Python can force a graph redraw, so I let it inform QML which then here forces a redraw :)
    function tellQMLToRedrawGraph2() {
        flickable.contentX=flickable.contentX -1 
        flickable.contentX=flickable.contentX + 1
    }
    
    
//*******************************************************
    
    
    id: mainPage
    tools: toolbar

    anchors.margins: 10


    Column {
	id: col
	spacing: 10
	width: parent.width   
	  
	  

	  

	Label {
	    id: lbltitle
	    text: "Angle graph"
	    font.bold: true
	    font.pixelSize: 32
	}


	Flickable {  
	    id: flickable
	    width: 1
	    height: 1
	    clip: true
	    boundsBehavior: Flickable.StopAtBounds
	    
	    AngleGraph {
		id: angleGraph01
		width: 1
		height: 1  
		clip: true
	    }
	    contentWidth: angleGraph01.width; contentHeight: angleGraph01.height	      
	}
	  
	  
	  
	  


	Row{
	    anchors.top: AngleGraph.bottom
	    Text {
		text: "Legend: "
		font.bold: true
		font.pixelSize: 18
	    }    
	    Rectangle{
		width: 16
		height: 16
		radius: 8
		color: "yellow"
	    }
	    Text {
		text: " Portrait angle, "
		font.pixelSize: 18
	    }
	    Rectangle{
		width: 16
		height: 16
		radius: 8
		color: "green"
	    }
	    Text {
		text: " Landscape angle"
		font.pixelSize: 18
	    }
	}

    }




//*******************************************************



    ToolBarLayout {
	id: toolbar
	
	ToolButtonRow { } //needed, so the menu is on the right side	      
	    
	ToolIcon {
	    iconId: "toolbar-view-menu" ;
	    onClicked: myMenu.open();
	}
    }    
   
}
